@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /* gifsicle.c - gifsicle's main loop.
-   Copyright (C) 1997-2009 Eddie Kohler, ekohler@cs.ucla.edu
+   Copyright (C) 1997-2010 Eddie Kohler, ekohler@cs.ucla.edu
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -1484,7 +1484,8 @@ main(int argc, char *argv[])
       break;
 
      case CROP_TRANSPARENCY_OPT:
-      if (clp->negated) goto no_crop_transparency;
+      if (clp->negated)
+	goto no_crop_transparency;
       def_frame.crop = copy_crop(def_frame.crop);
       def_frame.crop->transparent_edges = 1;
       break;
@@ -1763,7 +1764,7 @@ main(int argc, char *argv[])
 #else
       printf("LCDF Gifsicle %s\n", VERSION);
 #endif
-      printf("Copyright (C) 1997-2009 Eddie Kohler\n\
+      printf("Copyright (C) 1997-2010 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
